<?php

/**
 * @file
 * Handles the database functions for managing user subscriptions
 */

/**
 * Functions for managing user subscriptions 
 */
class User_subscription_db {
	
	/**
	 * Save payment data when a user subscribes and pays. This stores limited non-sensitive data ab the payment for reference, info
	 *  about the subscription, and info connecting the merchant to drupal. Will add version to user subscription history table.
	 * 
	 * @param array $payment_info hash of payment info
	 *   - "role_subscription_id" id of subscription being subscribed to
	 *   - "first_name"
	 *   - "last_name"
	 *   - "phone"
	 * 
	 * @param object $subscription info about the subscription
	 *   - "id" role subscription id
	 * @param string merchant_id the id generated by the merchant that provides a link from this database to the payment engine (merchant)
	 * @param integer invoice_id generated by helper function, a random number with a timestamp to act as a link from the payment engine
	 *        to this database.
	 * 
	 * @return mixed a database query result resource, or FALSE if the query was not executed correctly.
	 */
	function save_payment($payment_info, $subscription, $merchant_id, $invoice_id){
		global $user;
		
		$query = "insert into {role_subscription_user_payments} (vid, uid, invoice_id, merchant_id) " .
			     "values ('%d', '%d', '%s', '%s');";
		
		$id = db_next_id('role_subscription_user_payments');//get next id for this table	   
		$vid = db_next_id('role_subscription_user_payments_history');//get next id for this table
		
		if (!$id OR !$vid) return;//fail
		
		$result = db_query($query, $vid, $user->uid, $invoice_id, $merchant_id);//add main user payment entry with values that won't change
			
		/**
		 * Add version
		 */					
		if ($result)
			User_subscription_history_db::add_version($payment_info, $id, $subscription->id, 'active');
		
		return $result;
	}
	
	/**
	 * Update payment info in local database. Will add version to user subscription history table.
	 * 
	 * @param array $payment_info form values for payment
	 *   - "first_name"
	 *   - "last_name"
	 *   - "company_name"
	 *   - "address1"
	 *   - "address2"
	 *   - "city"
	 *   - "state"
	 *   - "zip"
	 *   - "phone"
	 *   
	 * @param integer $id user subscription ID
	 * 
	 * @return mixed a database query result resource, or FALSE if the query was not executed correctly.
	 */
	function update_payment($payment_info, $id){
		/**
		 * Get role subscription ID
		 */
		if (!$role_subscription_id = self::find($id)->role_subscription_id)
			return;//fail

		$vid = db_next_id('role_subscription_user_payments_history');//get next id for this table
		
		/**
		 * Add a new version
		 */
		if (!User_subscription_history_db::add_version($payment_info, $id, $role_subscription_id, 'active'))
			return;//fail
		
		/**
		 * Update user payments to point to new version
		 */
		$query = "update {role_subscription_user_payments} set vid = '%d' where id = %d;";
			   
		return db_query($query, $vid, $id);
	}
	
	/**
	 * Cancel user subscription for ID. Will create a new version with "cancelled" status and point to that
	 *  new version.
	 * 
	 * @param integer $id user subscription ID
	 * 
	 * @todo (add suspend/activate functionality if payment engine supports it)
	 * 
	 * @return mixed a database query result resource, or FALSE if the query was not executed correctly.
	 */
	function cancel_subscription($id){
		$payment_info = (array) self::find($id);//get current payment info
		
		$vid = db_next_id('role_subscription_user_payments_history');//get next id for this table
		
		/**
		 * Add a new version as "cancelled"
		 */
		if (!User_subscription_history_db::add_version($payment_info, $id, $payment_info['role_subscription_id'], 'cancelled'))
			return;//fail
		
		/**
		 * Update user payments to point to new version
		 */
		$query = "update {role_subscription_user_payments} set vid = '%d' where id = %d;";
			   
		return db_query($query, $vid, $id);
	}
	
	/**
	 * Find all user subscriptions
	 * 
	 * @return mixed a database query result resource, or FALSE if the query was not executed correctly.
	 */			
	function find_all(){
		$query = "select rsp.*, rsh.title, rsph.role_subscription_id, rsph.phone, " .
				"CONCAT_WS(' ', rsph.first_name, rsph.last_name) as name, rsph.company_name, " .
				"u.name as username, u.mail from {role_subscription_user_payments} rsp " .
				"join {role_subscription_user_payments_history} rsph on rsp.vid = rsph.vid " .//join with current version				
				"join {role_subscription} rs on rs.id = rsph.role_subscription_id " .
				"join {role_subscription_history} rsh on rsh.vid = rs.vid " .
				"join {users} u on u.uid = rsp.uid " .
				"where rsph.status = 'active'; ";
	
		return db_query($query);
	}
	
	/**
	 * Find all user subscriptions for a role subscription
	 * 
	 * @param integer $id role subscription ID
	 * 
	 * @return mixed a database query result resource, or FALSE if the query was not executed correctly.
	 */
	function find_all_for_subscription($id){
		$query = "select rsp.*, rsh.title, u.name as username from {role_subscription_user_payments} rsp " .
				"join {role_subscription_user_payments_history} rsph on rsp.vid = rsph.vid " .//join with current version				
				"join {role_subscription} rs on rs.id = rsph.role_subscription_id " .
				"join {role_subscription_history} rsh on rsh.vid = rs.vid " .
				"join {users} u on u.uid = rsp.uid " .
				"where rsph.status = 'active' and rs.id = %d; ";
	
		return db_query($query, $id);
	}
	
	/**
	 * Helper function to get all user subscriptions for a role subscription in an array
	 * 
	 * @param integer $id role subscription ID
	 * 
	 * @return array list of user subscriptions for role subscription
	 */
	function get_all_for_subscription($id){
		$result = self::find_all_for_subscription($id);
		
		while ($row = db_fetch_object($result))
			$subscriptions [] = $row;
			
		return $subscriptions;
	}
	
	/**
	 * Find user subscription
	 * 
	 * @param integer $id ID of user subscription
	 * 
	 * @return object user subscription data
	 */
	function find($id){
		if (!$id) return;//fail
		
		$query = "select rsp.*, rsph.role_subscription_id, rsph.first_name, rsph.last_name, rsph.company_name, rsph.address1, " .
				"rsph.address2, rsph.city, rsph.state, rsph.zip, rsph.phone, " .
				"r.name as role, rsh.title, u.name as username, rs.role_subscription_group_id " .
				"from {role_subscription_user_payments} rsp " .
				"join {role_subscription_user_payments_history} rsph on rsp.vid = rsph.vid " .//join with current version
				"join {role_subscription} rs on rs.id = rsph.role_subscription_id " .
				"join {role_subscription_history} rsh on rsh.vid = rs.vid " .
				"join {users} u on u.uid = rsp.uid " .
				"join {role} r on r.rid = rs.rid " .
				"where rsph.status = 'active' and rsp.id = '%d';";
	
		return db_fetch_object(db_query($query, $id));
	}
	
	/**
	 * Find a user's subscriptions
	 * 
	 * @param integer $uid pass in user id
	 * 
	 * @return mixed a database query result resource, or FALSE if the query was not executed correctly.
	 */
	function find_all_for_user($uid){
		if (!$uid) return;//fail
		
		$query = "select rsp.*, rsph.role_subscription_id, rsph.first_name, rsph.last_name, rsph.company_name, rsph.address1, " .
				"rsph.address2, rsph.city, rsph.state, rsph.zip, rsph.phone, " .
				"rsh.title, u.name as username from {role_subscription_user_payments} rsp " .
				"join {role_subscription_user_payments_history} rsph on rsp.vid = rsph.vid " .//join with current version
				"join {role_subscription} rs on rs.id = rsph.role_subscription_id " .
				"join {role_subscription_history} rsh on rsh.vid = rs.vid " .
				"join {users} u on u.uid = rsp.uid " .
				"where rsph.status = 'active' and rsp.uid = %d; ";
		
		//echo str_replace(array('{', '}'), '', $query);die;
		
		return db_query($query, $uid);
	}
	
	/**
	 * Is a user currently subscription to a role subscription?
	 * 
	 * @param integer $uid user id
	 * @param integer $rs_id role subscription id
	 * 
	 * @return boolean TRUE if found, FALSE if not
	 */
	function user_subscribed_to_role_subscription($uid, $rs_id) {

		/**
		 * First get the role subscription group for this subscription level
		 */
		$group_id = db_fetch_object(db_query("select rs.role_subscription_group_id from {role_subscription} rs where rs.id = %d", $rs_id))->role_subscription_group_id;

		/**
		 * Then see if the user is already subscription to that subscription group
		 */
		$query = "select rs.role_subscription_group_id from {role_subscription_user_payments} rsp " .
				"join {role_subscription_user_payments_history} rsph on rsp.vid = rsph.vid " .//join with current version
				"join {role_subscription} rs on rs.id = rsph.role_subscription_id " .
				"join {role_subscription_history} rsh on rsh.vid = rs.vid " .
				"join {users} u on u.uid = rsp.uid " .
				"where rsph.status = 'active' and rsp.uid = %d and rs.role_subscription_group_id = %d; ";
		
		return db_fetch_object(db_query($query, $uid, $group_id)) ? TRUE : FALSE;
	}
	
	/**
	 * Get all user's subscriptions
	 * 
	 * @param integer $uid pass in user ID
	 * 
	 * @return array return associative array with role subscription ID's as the associative keys and the payment information
	 *   as the value of each array item.
	 */
	function get_all_user_subscriptions($uid){
		$result = self::find_all_for_user($uid);//get db result
	
		/**
		 * Build hash
		 */
		while ($_ = db_fetch_object($result))
			$user_subscriptions [$_->role_subscription_id] = $_;//assign role subscription as id
			
		return $user_subscriptions;
	}
	
	/**
	 * Change role subscription id for a user subscription (meaning that the user has chosen to upgrade/downgrade a subscription)
	 * 
	 * Steps
	 *   -find old version
	 *   -created new version
	 *   -update role subscription to point to new version
	 * 
	 * @param integer $id user subscription ID
	 * @param integer $role_subscription_id role subscription ID
	 * 
	 * @return mixed a database query result resource, or FALSE if the query was not executed correctly.
	 */
	function change_role_subscription($id, $role_subscription_id){
		if (!$id OR !$role_subscription_id) return;//fail
		
		/**
		 * Get old version ID for this subscription
		 */
		if (!$old_vid = self::find($id)->vid)
			return;//fail
		
		/**
		 * Get payment info for old version ID
		 */
		if (!$version_payment_info = (array) User_subscription_history_db::find($old_vid))
			return;//fail
		
		$vid = db_next_id('role_subscription_user_payments_history');//get next id for this table
		
		/**
		 * Add a new version
		 */
		if (!User_subscription_history_db::add_version($version_payment_info, $role_subscription_id, 'active'))	
			return;//fail
		
		/**
		 * Finally update role subscription to new version
		 */
		$query = "update {role_subscription_user_payments} set vid = %d where id = %d; ";
			   
		return db_query($query, $vid, $id);	
	}
}